/* ========================================================================= */
/* Run a SQL Query from SQL2Excel.txt                                        */
/* Jon Wolfers     May 2007                           www.sahananda.fwbo.net */
/* ========================================================================= */
/*                                                                           */
/* A menu system for performing SQL Queries and returning spreadsheets       */
/*                                                                           */
/* Menu entries are defined in SQLmenu.txt in the format                     */
/* Description {Query} (ParmName,ParmType[,parmDefault])...                  */
/*                                                                           */
/* Description appears on the menu                                           */
/* Query is a valid SQL query with replacable parms marked as ^1^, ^2^...    */
/* Each replacable parm has a definition pair of Name & Type.                */
/* Type can be from [Date,Number,String], Date and number are validated      */
/*                                                                           */
/* ALL returned columns must have a name ie: sum(takings) AS LOLLY           */
/*                                                                           */
/*                                                                           */
/* An example:                                                               */
/*Last Month sales {select * from sales where date > ^1^} (StartDate,Date,-1)*/
/*                                                                           */
/* ========================================================================= */
call on USER SQLError NAME SQLException         /* Set up SQL Error trapping */

MyDialog = .SQLMenuDialog~new                   /* Create  the menu dialog   */
MyDialog~Execute('ShowTop')                     /* Run     the menu dialog   */
MyDialog~DeInstall                              /* Destroy the menu dialog   */

exit                                            /* The end                   */

SQLException:                                   /* SQL Error Handler         */
/* ------------------------------------------------------------------------- */
call SQLError Condition("O")

/* ========================================================================= */
::requires 'oodwin32.cls'                                 /* Windows GUI     */
::REQUIRES 'SQLObj.frm'    /* Orexx/SQL from http://rexxsql.sourceforge.net/ */
/* ========================================================================= */
::class SQLMenuDialog subclass userdialog inherit AdvancedControls
/* ------------------------------------------------------------------------- */
::method Init
/* ------------------------------------------------------------------------- */
expose ControlArray

  self~Init:super

  parse source . . me                        /* gives name & path of program */
  mydir  = me~left(me~lastpos('\') - 1)                 /* where is code     */

  MyFile=.stream~new('c:\sqlmenu.txt')
  MyFile~open('read')
  ControlArray=MyFile~ArrayIn
  MyFile~Close

  if ControlArray~last = .nil
  then do
     call Errormessage 'Control file c:\sqlmenu.txt could not be found.'
     exit
  end /* DO */

  do i = 1 to controlArray~last
     if controlArray[i]~left(1) = '*' then controlArray~remove(i)
  end /* DO */
  controlArray = controlArray~makeArray            /* pack the control array */

                                             /* create windows dialog object */
  rc=self~CreateCenter(300,30+(20*ControlArray~Items), 'Run SQL Queries')

  self~InitCode=(rc=0)

/* ------------------------------------------------------------------------- */
::method DefineDialog
/* ------------------------------------------------------------------------- */
expose ControlArray

  bottomLineY = 10+(20 * ControlArray~Items)
                                                              /* status line */
  self~addText(10, bottomLineY + 3, 220, 9, 'Select a query', 'CENTER', 10)
                                                             /* Leave button */
  self~AddButton( 1, 240, bottomLineY, 50, 15, 'Leave', 'Ok', 'DEFAULT')

  button=11                                       /* Our buttons start at 11 */
  do ControlLine over ControlArray                /* For each control line   */
     parse var ControlLine label'{' .                       /* Add a button  */
     self~AddButton(button, 10, (20*(Button-10))-10, 50, 15, button-10,,
                                                             'Button_'||button)
     self~AddText(70,(20*(Button-10))-7,170,15,label~strip) /* And some text */
     button=button+1
  end /* DO */

/* ------------------------------------------------------------------------- */
::method Run
/* ------------------------------------------------------------------------- */
expose statusLine

  statusLine=Self~GetStaticControl(10)
  statusLine~SetColor(7,16) -- 7,
  statusFont = self~CreateFont('Arial',12,'BOLD')
  statusLine~setFont(statusFont,1)
  statusLine~Update

  self~run:super              /* this handles message calls - keypresses etc */

/* ------------------------------------------------------------------------- */
                                     /* Any button but OK will lead us here  */
::method UnKnown
/* ------------------------------------------------------------------------- */
expose controlArray statusLine
use arg msgname, args

  but_no=args~at(1)                          /* Which Button was pressed     */

  /* prevent further user interaction */
  do i = 11 to 11 + controlArray~items - 1
     button = self~GetButtonControl(i)
     if i = but_no
     then button~disable
     else button~hide
  end /* DO */

  definition = controlArray~at(but_no - 10)  /* control text for this button */

  parse var definition defTag '{'query'}' parms

  if .set~of('DELETE','UPDATE','INSERT')~hasIndex(query~word(1)~translate)
  then do
     call errormessage "You can only do select queries, not" query~word(1)
     self~OK
     return
  end /* DO */

  statusLine~setTitle('Initialising')

  /* parse out the replacable paramaters ----------------------------------- */
  parmno = 0
  pTagArr  = .array~new
  pTypeArr = .array~new
  pvalArr  = .array~new
  do while parms \= ''
     parmNo = parmNo + 1
     parse var parms .'('parmdata')' parms
     parse var parmdata tag','type','val
     pTagArr[parmNo]=tag
     if type~translate = 'DATE'   /* date may be specified relative to today */
     then if val~strip~left(1)~verify('+-') = 0
          then pValArr[parmNo] = date('s',date('b') + val,'b','-')
          else pValArr[parmNo] = val
     else pValArr[parmNo] = val
     pTypeArr[parmNo] = type~translate
  end /* DO */

  /* prompt user for replacable paramater values --------------------------- */
  do until valid?
     if parmNo > 0
     then pValArr = multiInputBox(defTag,'SQL Query',pTagArr,pValArr)


     if pValArr = .nil                                /* user pressed cancel */
     then do
        self~ok                                       /* clear the dialog    */
        return                                        /* leave               */
     end /* DO */

     /* validate parameter values ------------------------------------------ */
     valid? = .true                 /* assume parms OK till we see otherwise */
     do i = 1 to pValArr~items
        select
           when pTypeArr[i] = 'NUMBER'
              then if \pValArr[i]~datatype('n')
                   then do
                      call errormessage pTagArr[i] ,
                                          'should be a number - not' pValArr[i]
                      valid? = .false
                   end /* DO */
                   else query = query~changeStr('^'||i||'^',pValArr[i])
           when pTypeArr[i] = 'DATE'
                then if pValArr[i]~length \= 10 ,
                     |  pValArr[i]~ChangeStr('-','')~length \= 8 ,
                     |  \pvalArr[i]~ChangeStr('-','')~datatype('w') ,
                     |  pvalArr[i]~substr(5,1) \= '-' ,
                     |  pvalArr[i]~substr(8,1) \= '-' ,
                     then do
                        call errormessage pTagArr[i] 'should be a date ',
                                                  'YYYY-MM-DD - not' pValArr[i]
                        valid? = .false
                     end /* DO */
                     else query = query~changeStr('^'||i||'^',pValArr[i])
           otherwise
              query = query~changeStr('^'||i||'^',pValArr[i])
        end /* select */
     end /* DO */
  end

  say query~space
  statusLine~setTitle('Connecting to Database')

  /* Get a connection to SQL database ----------------------------------- */
  host = 'LOCALHOST'
  pwd  =  'xxxxxx'
  database = 'Accounts'
  MyCon = .SQLConnection~new(host, pwd, database)

  /* Test the connection ------------------------------------------------ */
  if MyCon <> .nil
  then do
     /* Check that we are connected to the right db */
     MyServer=.SQLResultTable~new(MyCon,"Select db_name(db_id()) as dbName")
     db_name. = MyServer~fetch
     if db_Name. \= .nil
     then if db_name.dbname <> database
          then do
             call errormessage 'Could not Connect to' database
             drop Myserver MyCon
             self~OK
             return
          end /* DO */
     statusLine~setTitle('You are connected to' db_name.dbname)
     drop MyServer
  end
  else do
     call errormessage 'Could not connect to' database
     self~OK
     return
  end /* DO */

  /* retrieve the query data ----------------------------------------------- */
  statusLine~setTitle('Please Wait - Running Query')
  ResultSet = .SQLResultTable~new(MyCon,query)~~Open
  ResArr = ResultSet~FetchRowSet
  statusLine~setTitle('Putting the data in the spreadsheet')
  if ResArr \= .nil
  then do
     /* Get a spreadsheet object to contain the report --------------------- */
     xl=.OleObject~new("Excel.Application")       /* create Excel Spreadsheet*/
     xl~Visible = .True              /* change to .false to do in background */
     xl~Application~DisplayAlerts = .true           /* Show save dialog etc. */
     xl~Application~SheetsInNewWorkbook = 1
     xl~WorkBooks~Add

     colTbl = .table~new        /* Table to map field names -> excel columns */
     typeTbl = .table~new       /* Table to map excel columns -> data types  */
     maxCol = 0

     info. = ResultSet~describe                    /* get column information */
     do tail over info.
        parse var tail .'.'infotype'.'infocol
        if infocol = 0 then iterate    /* stem.0 contains no of columns only */
        maxCol = max(maxCol,infoCol)

        select
           when infotype = 'NAME'
              then do
                 colTbl~put(infocol,info.tail)
                 xl~Cells(1,infoCol)~value = info.tail      /* column header */
              end /* DO */
  /* *********************************************************************** */
  /*                                                                         */
  /*                 Microsoft SQL Server Data types                         */
  /*                                                                         */
  /*[bigint,int,smallint,tinyint,money,smallmoney,datetime,smalldatetime,bit */
  /* timestamp,uniqueIdentifier,float,real,decimal(p,s),numeric(p,s),binary  */
  /* varBinary(n),char(n),nchar(n),nvarchar(n),varchar(n),text,ntext,image  ]*/
  /* *********************************************************************** */
           when infoType = 'TYPE'
              then do
                 typeTbl~put(info.tail~translate~strip,infoCol)
                 select
                      when info.tail~right(5) = 'MONEY'
                         then do
                            xl~Columns(infoCol":"infoCol)~Select
                            xl~Selection~Style = "Currency"
                         end /* DO */
                      otherwise
                         nop
                   end /* select */
              end
           otherwise
                /* other descriptions are [size, scale, precision, nullable] */
             nop
        end /* select */
     end /* DO */

     if resArr~last = .nil
     then do
        call errormessage 'There were no result rows!'
        lastResult = 0
     end /* DO */
     else lastResult = resArr~last

     do thisRow = 1 to lastResult
        s. = resArr[thisRow]
        do field over s.
           thisCol = colTbl~at(field)
           /* At this point data could be postprocessed if required */
           xl~cells(thisRow + 1, thisCol)~value = thisValue
        end /* DO */
     end /* DO */

  end /* DO */


  drop MyCon ResultSet

  lastcell = xl~ActiveCell~SpecialCells(xl~getconstant('XLLASTCELL'))~Address
  parse var lastcell '$'max_col'$'max_row
  if maxcol > 0  then xl~Columns("A:"||max_Col)~EntireColumn~AutoFit

  self~OK                                  /* Clear Dialog & leave           */

